#
#   5B - Playing screen
#

import os
import pygame
from pygame import Rect
from pygame.locals import *
from albow.widget import Widget
from albow.controls import Label, Row
from albow.dialogs import ask
from albow.resource import get_image, get_font
from albow.sound import pause_sound, resume_sound
from screen import Screen
from level_view import LevelView
import settings
from settings import frames_to_seconds
from game import filename_to_level_name

footer_font = get_font(18, "Vera.ttf")
level_name_font = get_font(18, "VeraBd.ttf")
level_name_color = (255, 255, 255)
message_font = get_font(18, "VeraBd.ttf")
message_color = (255, 255, 255)
time_font = get_font(18, "VeraBd.ttf")
tyger_sniggering_image = get_image("tyger_sniggering.png")
tyger_thwarted_image = get_image("tyger_thwarted.png")
tyger_gotcha_image = get_image("tyger_gotcha.png")

#---------------------------------------------------------------------------

class LevelScreen(Screen):

	def __init__(self, shell):
		Screen.__init__(self, shell)
		sw, sh = settings.screen_size
		game = shell.game
		level_name_view = LevelNameDisplay(game)
		clock_view = TimeDisplay(game, 'current_time', (255, 255, 255))
		deadline_view = TimeDisplay(game, 'deadline', (255, 128, 0))
		def fl(text):
			return Label(text, font = footer_font, fg_color = (255, 255, 0))
		items = [
			fl("Level"),
			level_name_view,
			fl("Time"),
			clock_view,
			fl("Deadline"),
			deadline_view,
		]
		footer = Row(items, width = sw, expand = level_name_view)
		#print "play: footer height =", footer.rect.height ###
		footer.rect.bottomleft = (0, sh)
		self.level_view = PlayingView(
			Rect((0, 0), (sw, sh - footer.rect.height)), shell)
		self.add(self.level_view)
		self.add(footer)
		self.level_view.focus()
	
	def leave_screen(self):
		pause_sound()
	
	def enter_screen(self):
		resume_sound()

	def key_down(self, e):
		key = e.key
		if key == K_ESCAPE:
			level = self.game.current_level
			if level.testing:
				level.reset()
				level.testing = False
				self.shell.show_editor_level()
			else:
				self.shell.show_menu()
		elif key == K_SPACE:
			self.space_key()
		else:
			Screen.key_down(self, e)
	
	def space_key(self):
		level = self.game.current_level
		if level.succeeded():
			if not level.testing:
				self.shell.next_level()
			else:
				self.shell.show_editor_level()
		elif level.failed():
			self.shell.start_level()
	
	def begin_frame(self):
		level = self.game.current_level
		level.begin_frame()
		if level.auto_end():
			level.reset()
			self.shell.show_menu()
		self.invalidate()

#---------------------------------------------------------------------------

class LevelNameDisplay(Widget):

	font = level_name_font

	def __init__(self, game):
		Widget.__init__(self, Rect((0, 0), self.font.size("X")))
		self.game = game
	
	def draw(self, surf):
		#self.draw_message(surf) or
		self.draw_level_name(surf)

	def draw_level_name(self, surf):
		path = self.game.current_level.pathname
		if path:
			name = filename_to_level_name(os.path.basename(path))
			buf = self.font.render(name, True, level_name_color)
			surf.blit(buf, (0, 0))

#---------------------------------------------------------------------------

def twelve_hour(hm):
	h = hm // 60
	m = hm % 60
	if h < 12:
		ap = "AM"
	else:
		h -= 12
		ap = "PM"
	if h == 0:
		h = 12
	return "%2d:%02d" % (h, m), ap

#---------------------------------------------------------------------------

class TimeDisplay(Widget):

	font = time_font
	
	def __init__(self, game, attr, color):
		w1, h1 = self.font.size("99:99")
		#w2, h2 = self.font.size(" ")
		w2 = h1 // 8
		w3, h3 = self.font.size("AM")
		r = Rect((0, 0), (w1 + w2 + w3, h1))
		Widget.__init__(self, r)
		self.game = game
		self.attr = attr
		self.fg_color = color
		self.ap_width = w3
	
	def draw(self, surf):
		time_in_frames = getattr(self.game.current_level, self.attr)
		if time_in_frames > -1:
			frame = surf.get_rect()
			time = frames_to_seconds(time_in_frames)
			text, ap = twelve_hour(time)
			buf = self.font.render(text, True, self.fg_color)
			surf.blit(buf, (0, 0))
			buf = self.font.render(ap, True, self.fg_color)
			surf.blit(buf, (frame.right - self.ap_width, 0))

#---------------------------------------------------------------------------

class PlayingView(LevelView):

	focus_finger = get_image("focus_finger.png")
	
	def __init__(self, rect, shell):
		self.shell = shell
		LevelView.__init__(self, rect, shell.game)

	def draw_with_transform(self, surf, t, with_selection):
		LevelView.draw_with_transform(self, surf, t, with_selection)
		level = self.level
		part = level.focused_part
		if part:
			r = part.get_translated_global_rect(t)
			x, y = r.bottomright
			surf.blit(self.focus_finger, (x - 20, y - 20))
		if level.tyger_thwarted():
			self.draw_tyger(surf, tyger_thwarted_image)
		elif level.tyger_active():
			if level.tyger_succeeded():
				self.draw_tyger(surf, tyger_gotcha_image)
			else:
				self.draw_tyger(surf, tyger_sniggering_image)
	
	def draw_tyger(self, surf, image):
		frame = surf.get_rect()
		r = image.get_rect()
		r.left = frame.left + 10
		r.bottom = frame.bottom - 10
		surf.blit(image, r)

	def mouse_down(self, e):
		p = self.mouse_to_global(e)
		#part = self.find_clicked_item(e, parts_only = True)
		part = self.level.find_part(p)
		if part and part.selectable_at_play_time():
			self.level.focused_part = part

	def key_down(self, e):
		level = self.level
		override = e.meta and e.shift
		tutorial = level.is_tutorial
		no_deadline = level.deadline < 0
		no_restriction = override or tutorial or no_deadline
		def confirm(mess):
			if tutorial or no_restriction:
				return True
			else:
				return ask(mess, ["OK", "Cancel"]) == "OK"
		key = e.key
		if key == K_w:
			self.wait_until_deadline()
			return
		if not level.testing:
			if key == K_p:
				if confirm("Return to previous level?"):
					self.shell.prev_level()
				return
			if key == K_n:
				if no_restriction:
					self.shell.next_level()
				return
		if key == K_r:
			if confirm("Restart level?"):
				self.shell.start_level()
		if key == K_SPACE:
			if tutorial or no_deadline:
				return
		part = level.focused_part
		if part and part.key_down(e):
			return
		LevelView.key_down(self, e)
	
	def wait_until_deadline(self):
		if self.level.has_tyger_script():
			if ask("Wait until deadline?", ["OK", "Cancel"]) == "OK":
				self.level.wait_until_deadline()
	
	def draw(self, surf):
		LevelView.draw(self, surf)
		self.draw_message(surf)

	def draw_message(self, surf):
		mess = ""
		level = self.level
		if level.is_tutorial:
			mess = "R: Restart   N: Next Level   P: Previous Level"
		elif level.deadline < 0:
			mess = "N: Next Level"
		elif level.succeeded():
			mess = "Press Space for Next Level"
		elif level.failed():
			mess = "Press Space to Restart Level"
		if mess:
			frame = surf.get_rect()
			buf = message_font.render(mess, True, message_color)
			r = buf.get_rect()
			x, y = frame.midbottom
			r.midbottom = (x, y - 5)
			surf.blit(buf, r)
			return True
