#
#   5B - Wire
#

from pygame import draw, Rect
from vector_algebra import Vector2, Point2, point_along_line

wire_colors = [
	("Red", (255, 0, 0)),
	("Orange", (255, 128, 0)),
	("Yellow", (255, 255, 0)),
	("Green", (0, 128, 0)),
	("Blue", (0, 128, 255)),
	("Cyan", (0, 255, 255)),
	("Purple", (128, 0, 255)),
	("Black", (64, 64, 64)),
	("Brown", (128, 64, 0)),
	("White", (255, 255, 255)),
]

wire_width = 4
bare_wire_width = 3
bare_wire_color = (96, 96, 96)
bare_wire_length = 5

selection_handle_size = (8, 8)

def num_wire_colors():
	return len(wire_colors)

def get_wire_color(no):
	return wire_colors[no][1]

#----------------------------------------------------------------------------

class Wire(object):
	#  color_no  int
	#  corners   [Point2]

	width = wire_width

	def __init__(self, color_no = 0, initial_point = None):
		self.color_no = color_no
		self.corners = []
		if initial_point:
			self.corners.append(WirePoint(self, initial_point))
	
	def num_points(self):
		return len(self.corners)
	
	def draw(self, surf, t):
		corners = self.corners
		d = Vector2(1, 1)
		p = [(t * wp.point - d) for wp in corners]
		#print "wire: draw:", p ###
		if len(p) > 1:
			#if corners[0].is_connected():
			e = p[0]
			p[0] = point_along_line(e, p[1], bare_wire_length)
			draw.line(surf, bare_wire_color, e.coords, p[0].coords, bare_wire_width)
			#if corners[-1].is_connected():
			e = p[-1]
			p[-1] = point_along_line(e, p[-2], bare_wire_length)
			draw.line(surf, bare_wire_color, e.coords, p[-1].coords, bare_wire_width)
			color = self.get_color()
			q = [c.coords for c in p]
			draw.lines(surf, color, False, q, self.width)
	
	def get_color(self):
		return get_wire_color(self.color_no)
	
	def extend_to_point(self, p, end = -1):
		if end:
			prev = -2
		else:
			prev = 1
		pts = self.corners
		if pts and pts[end].point == p \
			or len(pts) >= 2 and p.aligns_with(pts[end].point, pts[prev].point):
				pts[end].point = p
				return pts[end]
		else:
			wp = WirePoint(self, p)
			if end:
				pts.append(wp)
			else:
				pts.insert(0, wp)
			return wp
	
	def find_point(self, p):
		for i, wp in enumerate(self.corners):
			if p == wp.point:
				return wp
		return None
	
	def disconnect(self):
		for wp in self.corners:
			wp.disconnect()
	
	def extend_connected_endpoints(self): #, fork_points):
		for i in (0, -1):
			wp = self.corners[i]
			part = wp.connected_part
			if part:
				terminal_no = wp.connected_terminal_no
				p = part.global_terminal_position(terminal_no)
#				if p <> wp.point:
#					if fork_points:
#						wp.disconnect()
#						wp = self.extend_to_point(p, i)
#						wp.connect_to_terminal(part, terminal_no)
#					else:
				wp.point = p

	def create_from_file(self, r):
		wire = Wire()
		wire.read_from_file(r)
		return wire
	
	create_from_file = classmethod(create_from_file)
	
	def read_from_file(self, r):
		self.color_no = r.get_arg(int)
		for coords in r.get_tuple_list(int, int):
			wp = WirePoint(self, Point2(coords))
			self.corners.append(wp)
		r.next_line()

	def write_to_file(self, f):
		coords = [wp.point.coords for wp in self.corners]
		args = [self.color_no, coords]
		f.write("Wire %s\n" % args)

	def find_reachable_terminals(self, result):
		for i in (0, -1):
			wp = self.corners[i]
			part = wp.connected_part
			if part:
				part.find_reachable_terminals(wp.connected_terminal_no, result)
	
	def find_end(self, wp):
		corners = self.corners
		if corners:
			for end in (0, -1):
				if corners[end] is wp:
					return end

#----------------------------------------------------------------------------

class WirePoint(object):
	#  wire                   Wire
	#  point                  Point2
	#  connected_part         Part
	#  connected_terminal_no  int
	
	connected_part = None
	connected_terminal_no = None
	
	def __init__(self, wire, point):
		self.wire = wire
		self.point = point
	
	def is_connected(self):
		return bool(self.connected_part)
	
	def is_endpoint(self):
		return self.wire.find_end(self) is not None
	
	def delete(self):
		wire = self.wire
		if wire:
			wire.corners.remove(self)
			self.wire = None
		self.disconnect()

	def draw_highlight(self, surf, t, highlight_color):
		p = t * self.point
		r = Rect((0, 0), selection_handle_size)
		r.center = p.coords
		surf.fill(highlight_color, r)

	def offset(self, v, fork_points = False):
		if fork_points:
			self.duplicate()
		self.point += v

	def connect_to_terminal(self, part, terminal_no):
		#print "WirePoint: connecting", self, "to", part, "terminal", terminal_no ###
		self.connected_part = part
		self.connected_terminal_no = terminal_no
		part.connected_wire_points[terminal_no].add(self)

	def disconnect(self):
		part = self.connected_part
		if part:
			part.connected_wire_points[self.connected_terminal_no].remove(self)
			self.connected_part = None
			self.connected_terminal_no = None

	def extend_wire_to_point(self, p):
		#  Extends wire from this end and returns the new endpoint.
		#  If this point is not an endpoint of its wire, returns None.
		wire = self.wire
		end = wire.find_end(self)
		if end is not None:
			return wire.extend_to_point(p, end)

	def draw_rubber(self, surf, t, q):
		wire = self.wire
		color = wire.get_color()
		p = t * self.point
		d = Vector2(-1, -1)
		draw.line(surf, color, (p+d).coords, (q+d).coords, wire.width)

	def duplicate(self):
		wp = WirePoint(self.wire, self.point)
		if self.connected_part:
			part = self.connected_part
			n = self.connected_terminal_no
			self.disconnect()
			wp.connect_to_terminal(part, n)
		corners = self.wire.corners
		i = corners.index(self)
		if i == 0:
			corners.insert(0, wp)
		else:
			corners.insert(i + 1, wp)
